/*
 * Perform SPI communication using the X1600 SPI peripheral.
 *
 * Copyright (C) 2023, 2024 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/devices/spi-x1600.h>



/* Initialise a channel. */

Spi_x1600_channel::Spi_x1600_channel(l4_addr_t spi_start, l4_addr_t start,
                                     enum Clock_identifiers clock,
                                     Cpm_chip *cpm,
                                     Dma_channel *dma,
                                     enum Dma_x1600_request_type request_type,
                                     uint64_t frequency)
: Spi_channel(spi_start, start, clock, cpm, dma, request_type, frequency)
{
}



/* Initialise the peripheral abstraction. */

Spi_x1600_chip::Spi_x1600_chip(l4_addr_t spi_start, l4_addr_t start,
                               l4_addr_t end, Cpm_chip *cpm)
: Spi_chip(spi_start, start, end, cpm)
{
}

Spi_channel *
Spi_x1600_chip::_get_channel(uint8_t channel, Dma_channel *dma,
                             uint64_t frequency)
{
  (void) channel;

  // NOTE: Only sending is supported.

  return new Spi_x1600_channel(_spi_start, _start, Clock_ssi0,
                               _cpm, dma, Dma_request_ssi0_out, frequency);
}



/* C language interface. */

void *x1600_spi_init(l4_addr_t spi_start, l4_addr_t start, l4_addr_t end, void *cpm)
{
  return new Spi_x1600_chip(spi_start, start, end, static_cast<Cpm_chip *>(cpm));
}

void *x1600_spi_get_channel(void *spi, uint8_t channel, void *dma, uint64_t frequency)
{
  return static_cast<Spi_x1600_chip *>(spi)->get_channel(channel,
    static_cast<Dma_channel *>(dma), frequency);
}

uint32_t x1600_spi_send(void *channel, uint32_t bytes, const uint8_t data[])
{
  return static_cast<Spi_x1600_channel *>(channel)->send(bytes, data);
}

uint32_t x1600_spi_send_dc(void *channel, uint32_t bytes, const uint8_t data[],
                            const int dc[], uint8_t char_size, int big_endian)
{
  return static_cast<Spi_x1600_channel *>(channel)->send_dc(bytes, data, dc,
    char_size, big_endian);
}

uint32_t x1600_spi_send_units(void *channel, uint32_t bytes, const uint8_t data[],
                               uint8_t unit_size, uint8_t char_size, int big_endian)
{
  return static_cast<Spi_x1600_channel *>(channel)->send_units(bytes, data,
    unit_size, char_size, big_endian);
}

uint32_t x1600_spi_transfer(void *channel, l4_addr_t vaddr,
                             l4re_dma_space_dma_addr_t paddr,
                             uint32_t count, uint8_t unit_size,
                             uint8_t char_size)
{
  return static_cast<Spi_x1600_channel *>(channel)->transfer(vaddr, paddr,
    count, unit_size, char_size);
}

uint32_t x1600_spi_transfer_descriptor(void *channel, l4_addr_t vaddr,
                                        l4re_dma_space_dma_addr_t paddr,
                                        uint32_t count, uint8_t unit_size,
                                        uint8_t char_size, l4_addr_t desc_vaddr,
                                        l4re_dma_space_dma_addr_t desc_paddr)
{
  return static_cast<Spi_x1600_channel *>(channel)->transfer(vaddr, paddr,
    count, unit_size, char_size, desc_vaddr, desc_paddr);
}
